import java.util.Set;
import java.util.TreeSet;
import java.util.HashSet;

public class Alternative extends Regex {
	
	private final Regex r1, r2;
	private final int size;
	
	public Regex getR1() {
		return r1;
	}

	public Regex getR2() {
		return r2;
	}
	
	public Alternative(Regex r1, Regex r2) {
		super();
		this.r1 = r1;
		this.r2 = r2;
		this.size = r1.size() + r2.size() + 1;
	}

	@Override
	protected void appendTo(StringBuilder sb, int prec) {
		r1.toStringAux(sb, getPrecedence());
		sb.append('|');
		r2.toStringAux(sb, getPrecedence());
	}

	@Override
	protected int getPrecedence() {
		return 0;
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((r1 == null) ? 0 : r1.hashCode());
		result = prime * result + ((r2 == null) ? 0 : r2.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		Alternative other = (Alternative) obj;
		if (r1 == null) {
			if (other.r1 != null)
				return false;
		} else if (!r1.equals(other.r1))
			return false;
		if (r2 == null) {
			if (other.r2 != null)
				return false;
		} else if (!r2.equals(other.r2))
			return false;
		return true;
	}
	
	public int size() {
		return size;
	}
	
	public EpsilonNFA toEpsilonNFA(Set<Character> alpha) {
		EpsilonNFA n1 = r1.toEpsilonNFA(alpha);
		EpsilonNFA n2 = r2.toEpsilonNFA(alpha);
	    Set<Transition> ts = new HashSet<>();
	    Set<State> states = new HashSet<>();
	    Set<State> finalStates = new HashSet<>();
	    
	    ts.addAll(n1.transitions);
	    ts.addAll(n2.transitions);
	    states.addAll(n1.states);
	    states.addAll(n2.states);
	    finalStates.addAll(n1.finalStates);
	    finalStates.addAll(n2.finalStates);	 
	    
	    State start = new State();
	    states.add(start);
	    ts.add(new Transition (start,n1.startState,Transition.EPSILON));
	    ts.add(new Transition (start,n2.startState,Transition.EPSILON));
	    
		return new EpsilonNFA(states, ts, alpha, start, finalStates);
	}
	
}
